@@ -72,6 +72,7 @@ from .crypto import pw_encode_with_version_and_mac, pw_decode_with_version_and_m
 from .lnutil import ChannelBackupStorage
 from .lnchannel import ChannelBackup
 from .channel_db import UpdateStatus
+from .channel_db import get_mychannel_info, get_mychannel_policy
 from .submarine_swaps import SwapManager
 
 if TYPE_CHECKING:
@@ -381,7 +382,7 @@ class LNWorker(Logger, NetworkRetryManager[LNPeerAddr]):
         for srv_ans in srv_answers:
             try:
                 # note: this might block for several seconds
-                answers = dns.resolver.query(srv_ans['host'])
+                answers = dns.resolver.resolve(srv_ans['host'])
             except dns.exception.DNSException as e:
                 self.logger.info(f'failed querying (2) dns seed "{dns_seed}" for ln peers: {repr(e)}')
                 continue
@@ -944,12 +945,13 @@ class LNWallet(LNWorker):
         if not peer:
             raise Exception('Dropped peer')
         await peer.initialized
-        htlc = peer.pay(route=route,
-                        chan=chan,
-                        amount_msat=lnaddr.get_amount_msat(),
-                        payment_hash=lnaddr.paymenthash,
-                        min_final_cltv_expiry=lnaddr.get_min_final_cltv_expiry(),
-                        payment_secret=lnaddr.payment_secret)
+        htlc = peer.pay(
+            route=route,
+            chan=chan,
+            amount_msat=lnaddr.get_amount_msat(),
+            payment_hash=lnaddr.paymenthash,
+            min_final_cltv_expiry=lnaddr.get_min_final_cltv_expiry(),
+            payment_secret=lnaddr.payment_secret)
         util.trigger_callback('htlc_added', chan, htlc, SENT)
         payment_attempt = await self.await_payment(lnaddr.paymenthash)
         if payment_attempt.success:
@@ -1094,13 +1096,15 @@ class LNWallet(LNWorker):
                 path = full_path[:-len(private_route)]
             else:
                 # find path now on public graph, to border node
-                path = self.network.path_finder.find_path_for_payment(self.node_keypair.pubkey, border_node_pubkey, amount_msat,
-                                                                      my_channels=scid_to_my_channels)
+                path = self.network.path_finder.find_path_for_payment(
+                    self.node_keypair.pubkey, border_node_pubkey, amount_msat,
+                    my_channels=scid_to_my_channels)
             if not path:
                 continue
             try:
-                route = self.network.path_finder.create_route_from_path(path, self.node_keypair.pubkey,
-                                                                        my_channels=scid_to_my_channels)
+                route = self.network.path_finder.create_route_from_path(
+                    path, self.node_keypair.pubkey,
+                    my_channels=scid_to_my_channels)
             except NoChannelPolicy:
                 continue
             # we need to shift the node pubkey by one towards the destination:
@@ -1112,20 +1116,23 @@ class LNWallet(LNWorker):
                 short_channel_id = ShortChannelID(short_channel_id)
                 # if we have a routing policy for this edge in the db, that takes precedence,
                 # as it is likely from a previous failure
-                channel_policy = self.channel_db.get_policy_for_node(short_channel_id=short_channel_id,
-                                                                     node_id=prev_node_id,
-                                                                     my_channels=scid_to_my_channels)
+                channel_policy = self.channel_db.get_policy_for_node(
+                    short_channel_id=short_channel_id,
+                    node_id=prev_node_id,
+                    my_channels=scid_to_my_channels)
                 if channel_policy:
                     fee_base_msat = channel_policy.fee_base_msat
                     fee_proportional_millionths = channel_policy.fee_proportional_millionths
                     cltv_expiry_delta = channel_policy.cltv_expiry_delta
                 node_info = self.channel_db.get_node_info_for_node_id(node_id=node_pubkey)
-                route.append(RouteEdge(node_id=node_pubkey,
-                                       short_channel_id=short_channel_id,
-                                       fee_base_msat=fee_base_msat,
-                                       fee_proportional_millionths=fee_proportional_millionths,
-                                       cltv_expiry_delta=cltv_expiry_delta,
-                                       node_features=node_info.features if node_info else 0))
+                route.append(
+                    RouteEdge(
+                        node_id=node_pubkey,
+                        short_channel_id=short_channel_id,
+                        fee_base_msat=fee_base_msat,
+                        fee_proportional_millionths=fee_proportional_millionths,
+                        cltv_expiry_delta=cltv_expiry_delta,
+                        node_features=node_info.features if node_info else 0))
                 prev_node_id = node_pubkey
             # test sanity
             if not is_route_sane_to_use(route, amount_msat, decoded_invoice.get_min_final_cltv_expiry()):
@@ -1138,12 +1145,14 @@ class LNWallet(LNWorker):
             if full_path:  # user pre-selected path
                 path = full_path
             else:  # find path now
-                path = self.network.path_finder.find_path_for_payment(self.node_keypair.pubkey, invoice_pubkey, amount_msat,
-                                                                      my_channels=scid_to_my_channels)
+                path = self.network.path_finder.find_path_for_payment(
+                    self.node_keypair.pubkey, invoice_pubkey, amount_msat,
+                    my_channels=scid_to_my_channels)
             if not path:
                 raise NoPathFound()
-            route = self.network.path_finder.create_route_from_path(path, self.node_keypair.pubkey,
-                                                                    my_channels=scid_to_my_channels)
+            route = self.network.path_finder.create_route_from_path(
+                path, self.node_keypair.pubkey,
+                my_channels=scid_to_my_channels)
             if not is_route_sane_to_use(route, amount_msat, decoded_invoice.get_min_final_cltv_expiry()):
                 self.logger.info(f"rejecting insane route {route}")
                 raise NoPathFound()
@@ -1323,7 +1332,7 @@ class LNWallet(LNWorker):
                 continue
             chan_id = chan.short_channel_id
             assert isinstance(chan_id, bytes), chan_id
-            channel_info = self.channel_db.get_channel_info(chan_id, my_channels=scid_to_my_channels)
+            channel_info = get_mychannel_info(chan_id, scid_to_my_channels)
             # note: as a fallback, if we don't have a channel update for the
             # incoming direction of our private channel, we fill the invoice with garbage.
             # the sender should still be able to pay us, but will incur an extra round trip
@@ -1333,8 +1342,7 @@ class LNWallet(LNWorker):
             cltv_expiry_delta = 1  # lnd won't even try with zero
             missing_info = True
             if channel_info:
-                policy = self.channel_db.get_policy_for_node(channel_info.short_channel_id, chan.node_id,
-                                                             my_channels=scid_to_my_channels)
+                policy = get_mychannel_policy(channel_info.short_channel_id, chan.node_id, scid_to_my_channels)
                 if policy:
                     fee_base_msat = policy.fee_base_msat
                     fee_proportional_millionths = policy.fee_proportional_millionths
