@@ -266,7 +266,11 @@ class Network(Logger, NetworkRetryManager[ServerAddr]):
 
         self.asyncio_loop = asyncio.get_event_loop()
         assert self.asyncio_loop.is_running(), "event loop not running"
-        self._loop_thread = None  # type: threading.Thread  # set by caller; only used for sanity checks
+        try:
+            self._loop_thread = self.asyncio_loop._mythread  # type: threading.Thread  # only used for sanity checks
+        except AttributeError as e:
+            self.logger.warning(f"asyncio loop does not have _mythread set: {e!r}")
+            self._loop_thread = None
 
         assert isinstance(config, SimpleConfig), f"config should be a SimpleConfig instead of {type(config)}"
         self.config = config
@@ -347,12 +351,13 @@ class Network(Logger, NetworkRetryManager[ServerAddr]):
     def has_channel_db(self):
         return self.channel_db is not None
 
-    def maybe_init_lightning(self):
+    def init_channel_db(self):
         if self.channel_db is None:
             from . import lnrouter
             from . import channel_db
             self.channel_db = channel_db.ChannelDB(self)
             self.path_finder = lnrouter.LNPathFinder(self.channel_db)
+            self.channel_db.load_data()
 
     def start_gossip(self):
         if self.lngossip is None:
