@@ -29,25 +29,26 @@ import base64
 import time
 import hashlib
 from collections import defaultdict
-from typing import Dict
+from typing import Dict, Union, Sequence, List
 
 from urllib.parse import urljoin
 from urllib.parse import quote
 from aiohttp import ClientResponse
 
 from electrum import ecc, constants, keystore, version, bip32, bitcoin
-from electrum.bitcoin import TYPE_ADDRESS, is_new_seed, seed_type, is_any_2fa_seed_type
-from electrum.bip32 import (deserialize_xpub, deserialize_xprv, bip32_private_key, CKD_pub,
-                            serialize_xpub, bip32_root, bip32_private_derivation, xpub_type)
+from electrum.bip32 import BIP32Node, xpub_type
 from electrum.crypto import sha256
-from electrum.transaction import TxOutput
-from electrum.mnemonic import Mnemonic
+from electrum.transaction import PartialTxOutput, PartialTxInput, PartialTransaction, Transaction
+from electrum.mnemonic import Mnemonic, seed_type, is_any_2fa_seed_type
 from electrum.wallet import Multisig_Wallet, Deterministic_Wallet
 from electrum.i18n import _
 from electrum.plugin import BasePlugin, hook
-from electrum.util import NotEnoughFunds
-from electrum.storage import STO_EV_USER_PW
+from electrum.util import NotEnoughFunds, UserFacingException
+from electrum.storage import StorageEncryptionVersion
 from electrum.network import Network
+from electrum.base_wizard import BaseWizard, WizardWalletPasswordSetting
+from electrum.logging import Logger
+
 
 def get_signing_xpub(xtype):
     if not constants.net.TESTNET:
@@ -58,9 +59,8 @@ def get_signing_xpub(xtype):
         raise NotImplementedError('xtype: {}'.format(xtype))
     if xtype == 'standard':
         return xpub
-    _, depth, fingerprint, child_number, c, cK = bip32.deserialize_xpub(xpub)
-    xpub = bip32.serialize_xpub(xtype, c, cK, depth, fingerprint, child_number)
-    return xpub
+    node = BIP32Node.from_xkey(xpub)
+    return node._replace(xtype=xtype).to_xpub()
 
 def get_billing_xpub():
     if constants.net.TESTNET:
@@ -107,14 +107,23 @@ class TrustedCoinException(Exception):
 
 
 class ErrorConnectingServer(Exception):
-    pass
+    def __init__(self, reason: Union[str, Exception] = None):
+        self.reason = reason
+
+    def __str__(self):
+        header = _("Error connecting to {} server").format('TrustedCoin')
+        reason = self.reason
+        if isinstance(reason, BaseException):
+            reason = repr(reason)
+        return f"{header}:\n{reason}" if reason else header
 
 
-class TrustedCoinCosignerClient(object):
+class TrustedCoinCosignerClient(Logger):
     def __init__(self, user_agent=None, base_url='https://api.trustedcoin.com/2/'):
         self.base_url = base_url
         self.debug = False
         self.user_agent = user_agent
+        Logger.__init__(self)
 
     async def handle_response(self, resp: ClientResponse):
         if resp.status != 200:
@@ -129,27 +138,39 @@ class TrustedCoinCosignerClient(object):
         except:
             return await resp.text()
 
-    def send_request(self, method, relative_url, data=None):
+    def send_request(self, method, relative_url, data=None, *, timeout=None):
         network = Network.get_instance()
         if not network:
             raise ErrorConnectingServer('You are offline.')
         url = urljoin(self.base_url, relative_url)
         if self.debug:
-            print('%s %s %s' % (method, url, data))
+            self.logger.debug(f'<-- {method} {url} {data}')
         headers = {}
         if self.user_agent:
             headers['user-agent'] = self.user_agent
         try:
             if method == 'get':
-                return Network.send_http_on_proxy(method, url, params=data, headers=headers, on_finish=self.handle_response)
+                response = Network.send_http_on_proxy(method, url,
+                                                      params=data,
+                                                      headers=headers,
+                                                      on_finish=self.handle_response,
+                                                      timeout=timeout)
             elif method == 'post':
-                return Network.send_http_on_proxy(method, url, json=data, headers=headers, on_finish=self.handle_response)
+                response = Network.send_http_on_proxy(method, url,
+                                                      json=data,
+                                                      headers=headers,
+                                                      on_finish=self.handle_response,
+                                                      timeout=timeout)
             else:
                 assert False
         except TrustedCoinException:
             raise
         except Exception as e:
             raise ErrorConnectingServer(e)
+        else:
+            if self.debug:
+                self.logger.debug(f'--> {response}')
+            return response
 
     def get_terms_of_service(self, billing_plan='electrum-per-tx-otp'):
         """
@@ -208,7 +229,8 @@ class TrustedCoinCosignerClient(object):
             'otp': otp,
             'transaction': transaction
         }
-        return self.send_request('post', 'cosigner/%s/sign' % quote(id), payload)
+        return self.send_request('post', 'cosigner/%s/sign' % quote(id), payload,
+                                 timeout=60)
 
     def transfer_credit(self, id, recipient, otp, signature_callback):
         """
@@ -236,19 +258,21 @@ server = TrustedCoinCosignerClient(user_agent="Electrum/" + version.ELECTRUM_VER
 
 class Wallet_2fa(Multisig_Wallet):
 
+    plugin: 'TrustedCoinPlugin'
+
     wallet_type = '2fa'
 
-    def __init__(self, storage):
+    def __init__(self, db, storage, *, config):
         self.m, self.n = 2, 3
-        Deterministic_Wallet.__init__(self, storage)
+        Deterministic_Wallet.__init__(self, db, storage, config=config)
         self.is_billing = False
         self.billing_info = None
         self._load_billing_addresses()
 
     def _load_billing_addresses(self):
         billing_addresses = {
-            'legacy': self.storage.get('trustedcoin_billing_addresses', {}),
-            'segwit': self.storage.get('trustedcoin_billing_addresses_segwit', {})
+            'legacy': self.db.get('trustedcoin_billing_addresses', {}),
+            'segwit': self.db.get('trustedcoin_billing_addresses_segwit', {})
         }
         self._billing_addresses = {}  # type: Dict[str, Dict[int, str]]  # addr_type -> index -> addr
         self._billing_addresses_set = set()  # set of addrs
@@ -263,19 +287,19 @@ class Wallet_2fa(Multisig_Wallet):
         return not self.keystores['x2/'].is_watching_only()
 
     def get_user_id(self):
-        return get_user_id(self.storage)
+        return get_user_id(self.db)
 
     def min_prepay(self):
         return min(self.price_per_tx.keys())
 
-    def num_prepay(self, config):
+    def num_prepay(self):
         default = self.min_prepay()
-        n = config.get('trustedcoin_prepay', default)
+        n = self.config.get('trustedcoin_prepay', default)
         if n not in self.price_per_tx:
             n = default
         return n
 
-    def extra_fee(self, config):
+    def extra_fee(self):
         if self.can_sign_without_server():
             return 0
         if self.billing_info is None:
@@ -285,45 +309,54 @@ class Wallet_2fa(Multisig_Wallet):
             return 0
         if self.is_billing:
             return 0
-        n = self.num_prepay(config)
+        n = self.num_prepay()
         price = int(self.price_per_tx[n])
         if price > 100000 * n:
             raise Exception('too high trustedcoin fee ({} for {} txns)'.format(price, n))
         return price
 
-    def make_unsigned_transaction(self, coins, outputs, config, fixed_fee=None,
-                                  change_addr=None, is_sweep=False):
+    def make_unsigned_transaction(self, *, coins: Sequence[PartialTxInput],
+                                  outputs: List[PartialTxOutput], fee=None,
+                                  change_addr: str = None, is_sweep=False) -> PartialTransaction:
         mk_tx = lambda o: Multisig_Wallet.make_unsigned_transaction(
-            self, coins, o, config, fixed_fee, change_addr)
-        fee = self.extra_fee(config) if not is_sweep else 0
-        if fee:
+            self, coins=coins, outputs=o, fee=fee, change_addr=change_addr)
+        extra_fee = self.extra_fee() if not is_sweep else 0
+        if extra_fee:
             address = self.billing_info['billing_address_segwit']
-            fee_output = TxOutput(TYPE_ADDRESS, address, fee)
+            fee_output = PartialTxOutput.from_address_and_value(address, extra_fee)
             try:
                 tx = mk_tx(outputs + [fee_output])
             except NotEnoughFunds:
                 # TrustedCoin won't charge if the total inputs is
                 # lower than their fee
                 tx = mk_tx(outputs)
-                if tx.input_value() >= fee:
+                if tx.input_value() >= extra_fee:
                     raise
-                self.print_error("not charging for this tx")
+                self.logger.info("not charging for this tx")
         else:
             tx = mk_tx(outputs)
         return tx
 
-    def on_otp(self, tx, otp):
+    def on_otp(self, tx: PartialTransaction, otp):
         if not otp:
-            self.print_error("sign_transaction: no auth code")
+            self.logger.info("sign_transaction: no auth code")
             return
         otp = int(otp)
         long_user_id, short_id = self.get_user_id()
-        raw_tx = tx.serialize()
-        r = server.sign(short_id, raw_tx, otp)
+        raw_tx = tx.serialize_as_bytes().hex()
+        assert raw_tx[:10] == "70736274ff", f"bad magic. {raw_tx[:10]}"
+        try:
+            r = server.sign(short_id, raw_tx, otp)
+        except TrustedCoinException as e:
+            if e.status_code == 400:  # invalid OTP
+                raise UserFacingException(_('Invalid one-time password.')) from e
+            else:
+                raise
         if r:
-            raw_tx = r.get('transaction')
-            tx.update(raw_tx)
-        self.print_error("twofactor: is complete", tx.is_complete())
+            received_raw_tx = r.get('transaction')
+            received_tx = Transaction(received_raw_tx)
+            tx.combine_with_other_psbt(received_tx)
+        self.logger.info(f"twofactor: is complete {tx.is_complete()}")
         # reset billing_info
         self.billing_info = None
         self.plugin.start_request_thread(self)
@@ -349,10 +382,10 @@ class Wallet_2fa(Multisig_Wallet):
         billing_addresses_of_this_type[billing_index] = address
         self._billing_addresses_set.add(address)
         self._billing_addresses[addr_type] = billing_addresses_of_this_type
-        self.storage.put('trustedcoin_billing_addresses', self._billing_addresses['legacy'])
-        self.storage.put('trustedcoin_billing_addresses_segwit', self._billing_addresses['segwit'])
+        self.db.put('trustedcoin_billing_addresses', self._billing_addresses['legacy'])
+        self.db.put('trustedcoin_billing_addresses_segwit', self._billing_addresses['segwit'])
         # FIXME this often runs in a daemon thread, where storage.write will fail
-        self.storage.write()
+        self.db.write(self.storage)
 
     def is_billing_address(self, addr: str) -> bool:
         return addr in self._billing_addresses_set
@@ -360,29 +393,35 @@ class Wallet_2fa(Multisig_Wallet):
 
 # Utility functions
 
-def get_user_id(storage):
+def get_user_id(db):
     def make_long_id(xpub_hot, xpub_cold):
         return sha256(''.join(sorted([xpub_hot, xpub_cold])))
-    xpub1 = storage.get('x1/')['xpub']
-    xpub2 = storage.get('x2/')['xpub']
+    xpub1 = db.get('x1/')['xpub']
+    xpub2 = db.get('x2/')['xpub']
     long_id = make_long_id(xpub1, xpub2)
     short_id = hashlib.sha256(long_id).hexdigest()
     return long_id, short_id
 
-def make_xpub(xpub, s):
-    version, _, _, _, c, cK = deserialize_xpub(xpub)
-    cK2, c2 = bip32._CKD_pub(cK, c, s)
-    return serialize_xpub(version, c2, cK2)
+def make_xpub(xpub, s) -> str:
+    rootnode = BIP32Node.from_xkey(xpub)
+    child_pubkey, child_chaincode = bip32._CKD_pub(parent_pubkey=rootnode.eckey.get_public_key_bytes(compressed=True),
+                                                   parent_chaincode=rootnode.chaincode,
+                                                   child_index=s)
+    child_node = BIP32Node(xtype=rootnode.xtype,
+                           eckey=ecc.ECPubkey(child_pubkey),
+                           chaincode=child_chaincode)
+    return child_node.to_xpub()
 
 def make_billing_address(wallet, num, addr_type):
     long_id, short_id = wallet.get_user_id()
     xpub = make_xpub(get_billing_xpub(), long_id)
-    version, _, _, _, c, cK = deserialize_xpub(xpub)
-    cK, c = CKD_pub(cK, c, num)
+    usernode = BIP32Node.from_xkey(xpub)
+    child_node = usernode.subkey_at_public_derivation([num])
+    pubkey = child_node.eckey.get_public_key_bytes(compressed=True)
     if addr_type == 'legacy':
-        return bitcoin.public_key_to_p2pkh(cK)
+        return bitcoin.public_key_to_p2pkh(pubkey)
     elif addr_type == 'segwit':
-        return bitcoin.public_key_to_p2wpkh(cK)
+        return bitcoin.public_key_to_p2wpkh(pubkey)
     else:
         raise ValueError(f'unexpected billing type: {addr_type}')
 
@@ -418,19 +457,23 @@ class TrustedCoinPlugin(BasePlugin):
             return
         if wallet.can_sign_without_server():
             return
-        if not wallet.keystores['x3/'].get_tx_derivations(tx):
-            self.print_error("twofactor: xpub3 not needed")
+        if not wallet.keystores['x3/'].can_sign(tx, ignore_watching_only=True):
+            self.logger.info("twofactor: xpub3 not needed")
             return
         def wrapper(tx):
+            assert tx
             self.prompt_user_for_otp(wallet, tx, on_success, on_failure)
         return wrapper
 
+    def prompt_user_for_otp(self, wallet, tx, on_success, on_failure) -> None:
+        raise NotImplementedError()
+
     @hook
-    def get_tx_extra_fee(self, wallet, tx):
+    def get_tx_extra_fee(self, wallet, tx: Transaction):
         if type(wallet) != Wallet_2fa:
             return
         for o in tx.outputs():
-            if o.type == TYPE_ADDRESS and wallet.is_billing_address(o.address):
+            if wallet.is_billing_address(o.address):
                 return o.address, o.value
 
     def finish_requesting(func):
@@ -442,15 +485,17 @@ class TrustedCoinPlugin(BasePlugin):
         return f
 
     @finish_requesting
-    def request_billing_info(self, wallet: 'Wallet_2fa'):
+    def request_billing_info(self, wallet: 'Wallet_2fa', *, suppress_connection_error=True):
         if wallet.can_sign_without_server():
             return
-        self.print_error("request billing info")
+        self.logger.info("request billing info")
         try:
             billing_info = server.get(wallet.get_user_id()[1])
         except ErrorConnectingServer as e:
-            self.print_error('cannot connect to TrustedCoin server: {}'.format(repr(e)))
-            return
+            if suppress_connection_error:
+                self.logger.info(repr(e))
+                return
+            raise
         billing_index = billing_info['billing_index']
         # add segwit billing address; this will be used for actual billing
         billing_address = make_billing_address(wallet, billing_index, addr_type='segwit')
@@ -478,16 +523,16 @@ class TrustedCoinPlugin(BasePlugin):
 
     def make_seed(self, seed_type):
         if not is_any_2fa_seed_type(seed_type):
-            raise BaseException('unexpected seed type: {}'.format(seed_type))
-        return Mnemonic('english').make_seed(seed_type=seed_type, num_bits=128)
+            raise Exception(f'unexpected seed type: {seed_type}')
+        return Mnemonic('english').make_seed(seed_type=seed_type)
 
     @hook
     def do_clear(self, window):
         window.wallet.is_billing = False
 
-    def show_disclaimer(self, wizard):
-        wizard.set_icon(':icons/trustedcoin-wizard.png')
-        wizard.stack = []
+    def show_disclaimer(self, wizard: BaseWizard):
+        wizard.set_icon('trustedcoin-wizard.png')
+        wizard.reset_stack()
         wizard.confirm_dialog(title='Disclaimer', message='\n\n'.join(self.disclaimer_msg), run_next = lambda x: wizard.run('choose_seed'))
 
     def choose_seed(self, wizard):
@@ -519,15 +564,15 @@ class TrustedCoinPlugin(BasePlugin):
         assert is_any_2fa_seed_type(t)
         xtype = 'standard' if t == '2fa' else 'p2wsh'
         bip32_seed = Mnemonic.mnemonic_to_seed(seed, passphrase)
-        xprv, xpub = bip32_root(bip32_seed, xtype)
-        xprv, xpub = bip32_private_derivation(xprv, "m/", derivation)
-        return xprv, xpub
+        rootnode = BIP32Node.from_rootseed(bip32_seed, xtype=xtype)
+        child_node = rootnode.subkey_at_private_derivation(derivation)
+        return child_node.to_xprv(), child_node.to_xpub()
 
     @classmethod
     def xkeys_from_seed(self, seed, passphrase):
         t = seed_type(seed)
         if not is_any_2fa_seed_type(t):
-            raise BaseException('unexpected seed type: {}'.format(t))
+            raise Exception(f'unexpected seed type: {t}')
         words = seed.split()
         n = len(words)
         # old version use long seed phrases
@@ -555,12 +600,12 @@ class TrustedCoinPlugin(BasePlugin):
 
     def on_password(self, wizard, password, encrypt_storage, k1, k2):
         k1.update_password(None, password)
-        wizard.storage.set_keystore_encryption(bool(password))
-        if encrypt_storage:
-            wizard.storage.set_password(password, enc_version=STO_EV_USER_PW)
-        wizard.storage.put('x1/', k1.dump())
-        wizard.storage.put('x2/', k2.dump())
-        wizard.storage.write()
+        wizard.data['x1/'] = k1.dump()
+        wizard.data['x2/'] = k2.dump()
+        wizard.pw_args = WizardWalletPasswordSetting(password=password,
+                                                     encrypt_storage=encrypt_storage,
+                                                     storage_enc_version=StorageEncryptionVersion.USER_PASSWORD,
+                                                     encrypt_keystore=bool(password))
         self.go_online_dialog(wizard)
 
     def restore_wallet(self, wizard):
@@ -574,9 +619,9 @@ class TrustedCoinPlugin(BasePlugin):
         f = lambda x: self.restore_choice(wizard, seed, x)
         wizard.passphrase_dialog(run_next=f) if is_ext else f('')
 
-    def restore_choice(self, wizard, seed, passphrase):
-        wizard.set_icon(':icons/trustedcoin-wizard.png')
-        wizard.stack = []
+    def restore_choice(self, wizard: BaseWizard, seed, passphrase):
+        wizard.set_icon('trustedcoin-wizard.png')
+        wizard.reset_stack()
         title = _('Restore 2FA wallet')
         msg = ' '.join([
             'You are going to restore a wallet protected with two-factor authentication.',
@@ -595,34 +640,30 @@ class TrustedCoinPlugin(BasePlugin):
             self.create_keystore(wizard, seed, passphrase)
 
     def on_restore_pw(self, wizard, seed, passphrase, password, encrypt_storage):
-        storage = wizard.storage
         xprv1, xpub1, xprv2, xpub2 = self.xkeys_from_seed(seed, passphrase)
         k1 = keystore.from_xprv(xprv1)
         k2 = keystore.from_xprv(xprv2)
         k1.add_seed(seed)
         k1.update_password(None, password)
         k2.update_password(None, password)
-        storage.put('x1/', k1.dump())
-        storage.put('x2/', k2.dump())
-        long_user_id, short_id = get_user_id(storage)
+        wizard.data['x1/'] = k1.dump()
+        wizard.data['x2/'] = k2.dump()
+        long_user_id, short_id = get_user_id(wizard.data)
         xtype = xpub_type(xpub1)
         xpub3 = make_xpub(get_signing_xpub(xtype), long_user_id)
         k3 = keystore.from_xpub(xpub3)
-        storage.put('x3/', k3.dump())
-
-        storage.set_keystore_encryption(bool(password))
-        if encrypt_storage:
-            storage.set_password(password, enc_version=STO_EV_USER_PW)
-
-        wizard.wallet = Wallet_2fa(storage)
-        wizard.create_addresses()
-
+        wizard.data['x3/'] = k3.dump()
+        wizard.pw_args = WizardWalletPasswordSetting(password=password,
+                                                     encrypt_storage=encrypt_storage,
+                                                     storage_enc_version=StorageEncryptionVersion.USER_PASSWORD,
+                                                     encrypt_keystore=bool(password))
+        wizard.terminate()
 
     def create_remote_key(self, email, wizard):
-        xpub1 = wizard.storage.get('x1/')['xpub']
-        xpub2 = wizard.storage.get('x2/')['xpub']
+        xpub1 = wizard.data['x1/']['xpub']
+        xpub2 = wizard.data['x2/']['xpub']
         # Generate third key deterministically.
-        long_user_id, short_id = get_user_id(wizard.storage)
+        long_user_id, short_id = get_user_id(wizard.data)
         xtype = xpub_type(xpub1)
         xpub3 = make_xpub(get_signing_xpub(xtype), long_user_id)
         # secret must be sent by the server
@@ -630,7 +671,7 @@ class TrustedCoinPlugin(BasePlugin):
             r = server.create(xpub1, xpub2, email)
         except (socket.error, ErrorConnectingServer):
             wizard.show_message('Server not reachable, aborting')
-            wizard.terminate()
+            wizard.terminate(aborted=True)
             return
         except TrustedCoinException as e:
             if e.status_code == 409:
@@ -680,31 +721,28 @@ class TrustedCoinPlugin(BasePlugin):
                 self.request_otp_dialog(wizard, short_id, None, xpub3)
             else:
                 wizard.show_message(str(e))
-                wizard.terminate()
+                wizard.terminate(aborted=True)
         except Exception as e:
-            wizard.show_message(str(e))
-            wizard.terminate()
+            wizard.show_message(repr(e))
+            wizard.terminate(aborted=True)
         else:
             k3 = keystore.from_xpub(xpub3)
-            wizard.storage.put('x3/', k3.dump())
-            wizard.storage.put('use_trustedcoin', True)
-            wizard.storage.write()
-            wizard.wallet = Wallet_2fa(wizard.storage)
-            wizard.run('create_addresses')
+            wizard.data['x3/'] = k3.dump()
+            wizard.data['use_trustedcoin'] = True
+            wizard.terminate()
 
     def on_reset_auth(self, wizard, short_id, seed, passphrase, xpub3):
         xprv1, xpub1, xprv2, xpub2 = self.xkeys_from_seed(seed, passphrase)
-        if (wizard.storage.get('x1/')['xpub'] != xpub1 or
-                wizard.storage.get('x2/')['xpub'] != xpub2):
+        if (wizard.data['x1/']['xpub'] != xpub1 or
+                wizard.data['x2/']['xpub'] != xpub2):
             wizard.show_message(_('Incorrect seed'))
             return
         r = server.get_challenge(short_id)
         challenge = r.get('challenge')
         message = 'TRUSTEDCOIN CHALLENGE: ' + challenge
         def f(xprv):
-            _, _, _, _, c, k = deserialize_xprv(xprv)
-            pk = bip32_private_key([0, 0], k, c)
-            key = ecc.ECPrivkey(pk)
+            rootnode = BIP32Node.from_xkey(xprv)
+            key = rootnode.subkey_at_private_derivation((0, 0)).eckey
             sig = key.sign_message(message, True)
             return base64.b64encode(sig).decode()
 
@@ -717,12 +755,12 @@ class TrustedCoinPlugin(BasePlugin):
         self.request_otp_dialog(wizard, short_id, new_secret, xpub3)
 
     @hook
-    def get_action(self, storage):
-        if storage.get('wallet_type') != '2fa':
+    def get_action(self, db):
+        if db.get('wallet_type') != '2fa':
             return
-        if not storage.get('x1/'):
+        if not db.get('x1/'):
             return self, 'show_disclaimer'
-        if not storage.get('x2/'):
+        if not db.get('x2/'):
             return self, 'show_disclaimer'
-        if not storage.get('x3/'):
+        if not db.get('x3/'):
             return self, 'accept_terms_of_use'
